
# THIS FILE IS GENERATED FROM wofrywise SETUP.PY
short_version = '1.0.0'
version = '1.0.0'
full_version = '1.0.0.dev0+e6727ea'
git_revision = 'e6727eaa206f9fe0d537df1baf7afb29dd949273'
release = False

if not release:
    version = full_version
    short_version += ".dev"
